#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__ = "The Item type class."
__version__ = "1.0.2"

from zclass import ZClass

class ItemZClass(
	ZClass):
	"Specific methods to the Item object type."

	def importblock(self):
		"The various imports at the top of a .py file."
		block = open(self.template_directory+'import.tmpl', 'r').read()
		block = block % ('from Globals import MessageDialog\n',
						 'from Globals import default__class_init__',
						 'import OFS.SimpleItem\n',
						 self.catalog_aware()[0])
		return block
		
	def meta_types(self):
		"Return the meta_types tuple."
		return ""

	def additional_methods(self):
		"Return the manage_edit() method."
		block = open(self.template_directory+'manage_edit.tmpl', 'r').read()
		return block % (self.attriblist(1)+', ',
						self.setattriblist(),
						self.catalog_aware()[3],
						'%s',
						'% self.id')

	def additional_tuples(self):
		"Return the manage_option tuple."
		block = open(self.template_directory+'manage_options.tmpl', 'r').read()
		return block

	def additional_htmlfiles(self):
		"Return the manage_main HTMLFile string."
		return "\n\tmanage_main = HTMLFile('"+ self.lname +"Edit', globals())\n"

	def class_block(self):
		"Return the class block."
		block = open(self.template_directory+'class.tmpl', 'r').read()
		return block % (self.name,
						self.catalog_aware()[1],
						'\tOFS.SimpleItem.Item,',
						self.name,
						self.product_name,
						self.icon,
						self.catalog_aware()[4],
						self.properties(),
						self.additional_tuples(),
						self.meta_types(),
						self.subadds(),
						self.permissions(),
						self.name,
						self.lname,
						self.additional_htmlfiles(),
						self.attriblist(2),
						self.setattriblist(),
						self.catalog_aware()[2],
						self.additional_methods(),
						self.name)

	def dtml_edit(self):
		"Return an Edit.dtml text."
		block = open(self.dtml_directory+'Edit.dtml', 'r').read()
		attribmarkup = ""
		for attribute in self.attributes:
			block2 = open(self.dtml_directory+'Edit_tr.dtml', 'r').read()
			attribmarkup = attribmarkup + block2 % (attribute[0],
													attribute[0],
													attribute[1],
													attribute[0])
		return block % (attribmarkup)

	def write_dtmlmethods(self):
		"Write Index and Add dtml files to disk."
		# Create the classnameAdd.dtml
		file = open(self.product_directory+self.lname+'Add.dtml', 'w')
		file.write(self.dtml_add())
		file.close
		# Then the classnameIndex.dtml
		file = open(self.product_directory+self.lname+'Index.dtml', 'w')
		file.write(self.dtml_index())
		file.close

		if self.nice_redirect == 1: 
			# Create the classnameAdd2.dtml
			file = open(self.product_directory+self.lname+'Add2.dtml', 'w')
			file.write(self.dtml_add2())
			file.close

		# And finally, the classnameEdit.dtml
		file = open(self.product_directory+self.lname+'Edit.dtml', 'w')
		file.write(self.dtml_edit())
		file.close

