#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__     = "The python script makes a complete Zope Python Product."
__version__ = "2.0.7"


"""
This is how you describe your Python Product.  Notice that
you can have as many Classes, SubClasses and Attributes as
you feel like.

[<ProductName>,
 [image list],
 <CatalogNameorNone>,
 <NiceRedirectp>,
 <StandardHeadersp>,
 [<ClassName>, <classtype>, <classiconname>, <catalogawarep>, 
  [[<attributename>, <attributetype>, <attributedefaultvalueorNone>]],
  [<SubClassName>, <subclasstype>, <subclassiconname>,
   [attribute list], 
   [subclass list]]]]
"""

import string
import sys

from itemzclass import ItemZClass
from folderzclass import FolderZClass
from zopeproduct import ZopeProduct


def add_class(class_list,
			  print_prefix,
			  product_name,
			  product_directory,
			  template_directory,
			  dtml_directory,
			  catalog_name,
			  nice_redirect,
			  standard_headers):
	"""
	This function calls the necessary classes' methods to make
	the individual classes.  It accepts one list; [<ClassName>...]
	"""
	# We print with this prefix to make it look at bit nicer
	# to the user.  :)
	print_prefix = print_prefix + '\t'

	try: class_name = class_list[0]
	except:	class_name = None
	try: class_type = class_list[1]
	except:	class_type = None
	try: class_icon = class_list[2]
	except: class_icon = None
	try: class_catalogaware = class_list[3]
	except: class_catalogaware = None
	try: class_attributes = class_list[4]
	except: class_attributes = None

	# Tell the user which class we are creating.
	print print_prefix, class_name

	# What I'm doing here should probably (semantically, at least)
	# be in zopeproduct.py, but that would mean to replicate some of the
	# code in this file, thus, if I change the format of the class_list
	# I'd have to update the same code twice, and I really don't like that.

	# Now, what am I doing here?  We're printing the class structure of the
	# entire product into one file in the Product directory - called
	# class-structure.txt.  It will be nice to refer to this file when
	# inquiries about the structure, CatalogAwareness and attributes arrise.
	# No need for browsing all the code.

	file = open(product_directory+'/class-structure.txt', 'a')
	file.write(print_prefix + class_name)
	if class_catalogaware == 1:
		file.write(' - CatalogAware')
	if nice_redirect == 1:
		file.write(' - Nice redirect')
	if standard_headers == 1:
		file.write(' - standard_html_*')
	file.write('\n')
	for attribute in class_attributes:
		file.write(print_prefix + ' + ' + attribute[0] + '\n')
	file.close()

	class_subclasses = []
	# For an explanation of this for-loop, read the next comments
	# 20 or so lines down in this file.
	for x in range(5, 100):
		try:
			# Simply append the subclass, if it exist, to the
			# class_subclasses list.
			class_subclasses.append(class_list[x][0])
		except:
			pass

	if string.lower(class_type) == 'item':
		zclass = ItemZClass(class_name,
							class_icon,
							class_attributes,
							class_subclasses,
							class_catalogaware,
							product_name,
							product_directory,
							template_directory,
							dtml_directory,
							catalog_name,
							nice_redirect,
							standard_headers)
	elif string.lower(class_type) == 'folder':
		zclass = FolderZClass(class_name,
							  class_icon,
							  class_attributes,
							  class_subclasses,
							  class_catalogaware,
							  product_name,
							  product_directory,
							  template_directory,
							  dtml_directory,
							  catalog_name,
							  nice_redirect,
							  standard_headers)

	# Call the methods that "makes" the class.
	try:
		zclass.add_class()
	except:
		print print_prefix + "Something ugly has occurred.  The zclass instance wasn't registered."
		print print_prefix + "Exiting, leaving you with traceback:\n"
		import traceback
		type, val, tb = sys.exc_info()
		sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
		sys.exit()
	
	# If you have more than 95 classes on the same level, this for-loop
	# won't detect it.  To correct it, you could either up the 100 to
	# whatever you feel like, or maybe take another look at your class
	# structure.  I would suggest the latter.  :)
	for x in range(5, 100):
		# The reason we are starting at 5 is that the fifth item of
		# the list is the next class at same level.
		try:
			# If we actually have onboard another class at same level,
			# we'll just call ourselves with this class' list as argument.
			# Neat, huh?  ;)
			add_class(class_list[x],
					  print_prefix,
					  product_name,
					  product_directory,
					  template_directory,
					  dtml_directory,
					  catalog_name,
					  nice_redirect,
					  standard_headers)
		except:
			pass

def copying():
	"Print copyright notice."
	print """
    mk-zprod version """+str(__version__)+""", Copyright (C) 2000 Erik Enge <erik@esol.no>
    mk-zprod comes with ABSOLUTELY NO WARRANTY; see the file COPYING.
    This is free software, and you are welcome to redistribute it under
    certain conditions; see the file COPYING.
	"""

def make_product(product_list):
	"""
	This function controls the process of adding classes to the
	Product and to make the __init__.py file.  This function also
	processes the [<ProductName>...] list and calls add_class() with
	the apropriate [<ClassName>...] list.

	It accepts one list; [<ProductName>...].
	"""
	copying()
	template_directory = 'tmpl/'
	image_directory = 'img/'
	dtml_directory = 'dtml/'
	
	product_name = product_list[0]
	product_images = product_list[1]
	catalog_name = product_list[2]
	nice_redirect = product_list[3]
	standard_headers = product_list[4]
	product_classes = product_list[5]

	zproduct = ZopeProduct(product_name,
						   product_images,
						   template_directory,
						   image_directory)

	print "Product name:", zproduct.name
	zproduct.add_product()

	print_prefix = ''

	if standard_headers == 1:
		print "Remember to add the make_action() function."

	add_class(product_classes,
			  print_prefix,
			  zproduct.name,
			  zproduct.name+'/',
			  template_directory,
			  dtml_directory,
			  catalog_name,
			  nice_redirect,
			  standard_headers)

list = [] # define your list here.

make_product(list)





