#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__     = "Provides all classes and methods needed for a Zope Python Class."
__version__ = "2.0.5"

import string

from docstring import DocString

class ZClass(
	DocString):
	"This class contains all methods needed to create a Zope Python Class."
	
	def __init__(self,
				 name,
				 icon,
				 attributes,
				 subclasses,
				 catalogaware,
				 product_name,
				 product_directory,
				 template_directory,
				 dtml_directory,
				 catalog_name,
				 nice_redirect,
				 standard_headers):
		"Initialize the class instance."
		self.name = name
		self.icon = icon
		self.attributes = attributes
		self.subclasses = subclasses
		self.catalogaware = catalogaware
		self.lname = string.lower(self.name)
		self.product_directory = product_directory
		self.template_directory = template_directory
		self.dtml_directory = dtml_directory
		self.catalog_name = catalog_name
		self.product_name = product_name
		self.nice_redirect = nice_redirect
		self.standard_headers = standard_headers

	def attriblist(self, type):
		"""
		Return the attribute list.  Possible types are:

		              1, just a plain list, sorted.
					  2, the list with default values, sorted.
					  3, the same as 1, but with id, unsorted.
		"""
		block = ""
		list1 = ""
		list2 = ""

		if type == 1:
			for attribute in self.attributes:
				if attribute[2] is None:
					list1 = list1 + attribute[0] + ", "
				else:
					list2 = list2 + attribute[0] + ", "
			return list1 + list2[:-2] # Strip the last comma.
		elif type == 2:
			list1 = ""
			list2 = ""
			for attribute in self.attributes:
				if attribute[2] is None:
					list1 = list1 + attribute[0] + ", "
				else:
					list2 = list2 + attribute[0] + "='" + str(attribute[2]) + "', "
			return ", " + list1 + list2[:-2] # Strip the last comma.
		elif type == 3:
			block = "id, "
			for attribute in self.attributes:
				block = block + attribute[0] + ", "
			return block

	def catalog_aware(self):
		"Return a list with the import string and the superclass string."
		list = []
		if self.catalogaware == 1:
			list = ['import Products\n\n',
					'\n\tProducts.ZCatalog.CatalogAwareness.CatalogAware,\n',
					'\t\tself.index_object()\n\n',
					'\t\tself.reindex_object()\n',
					'\tdefault_catalog = \''+str(self.catalog_name)+'\'\n']
		else:
			list = ['',
					'\n',
					'\n',
					'\n',
					'',]
		return list

	def importblock(self):
		"The various imports at the top of a .py file."
		block = ""
		for subclass in self.subclasses:
			block = block + "import "+string.lower(subclass)+"\n"
		block = block + self.catalog_aware[0]
		return block[:-1] # Strip the last newline character.

	def manage_addblock(self):
		"Return the manage_add block at the top of a Zope Python Class."
		block = open(self.template_directory+'manage_add.tmpl', 'r').read()
		if self.nice_redirect == 1:
			additional_add = str(self.lname)+"Add2 = HTMLFile('"+self.lname+"Add2', globals())\n"
		else:
			additional_add = ""
		return block % (additional_add,
						self.name,
						self.lname,
						self.name,
						self.attriblist(3),
						self.name,
						self.name,
						self.attriblist(1))

	def properties(self):
		"Return the _properties tuple."
		block = "\n\t_properties = (\n"
		for attribute in self.attributes:
			block = block + "\t\t{'id':'"+ attribute[0]  +"', 'type':'"+ attribute[1] +"'},\n"
		return block + "\t\t)\n"

	def meta_types(self):
		"Return the meta_types tuple."
		if self.subclasses > []:
			block = "\n\tmeta_types = (\n"
			for subclass in self.subclasses:
				block = block + "\t\t{'name':'"+ subclass +"', 'action':'manage_add"+ subclass +"Form'},\n"
			return block + "\t\t)\n"
		else:
			return ""

	def subadds(self):
		"Return the add methods of the subclasses."
		if self.subclasses > []:
			block = "\n"
			for subclass in self.subclasses:
				block = block + "\tmanage_add"+ subclass +"Form = "+ string.lower(subclass) +".manage_add"+ subclass +"Form\n"
				block = block + "\tmanage_add"+ subclass +" = "+ string.lower(subclass) +".manage_add"+ subclass +"\n"
				if self.nice_redirect == 1:
					block = block + "\t"+string.lower(subclass)+"Add2 = "+string.lower(subclass)+"."+string.lower(subclass)+"Add2\n"
			return block
		else:
			return ""

	def setattriblist(self):
		"Return the self.x = x for every attribute x."
		if self.attributes is not None:
			block = "\n"
			for attribute in self.attributes:
				block = block + "\t\tself."+ attribute[0] +" = "+ attribute[0] +"\n"
			return block
		else:
			return ""

	def permissions(self):
		"Return an extra permission tuple for each subclass."
		if self.subclasses is not None:
			block = "\n"
			for subclass in self.subclasses:
				block = block + "\t\t('Add "+ subclass +"',\n"
				block = block + "\t\t ('manage_add"+ subclass +"', 'manage_add"+ subclass +"Form')),\n"
			return block[:-1] # Strip last newline character.
		else:
			return ""

	def additional_methods(self):
		"Returns additonal methods to the class."
		return ""

	def additional_tuples(self):
		"Returns additional tuples to the class."
		return ""

	def additional_htmlfiles(self):
		"Returns additonal methods to the class."
		return ""
		
	def class_block(self):
		"Return the class block."
		block = open(self.template_directory+'class.tmpl', 'r').read()
		return block % (self.name,
						'',
						self.name,
						self.product_name,
						self.icon,
						self.catalog_aware()[4],
						self.properties(),
						self.additional_tuples(),
						self.meta_types(),
						self.subadds(),
						self.permissions(),
						self.name,
						self.lname,
						self.additional_htmlfiles(),
						self.attriblist(2),
						self.setattriblist(),
						self.additional_methods())

	def compile_class(self):
		"The top of the .py file."
		block = open(self.template_directory+'top.tmpl', 'r').read()
		return block % (self.docheader(),
						self.importblock(),
						self.manage_addblock(),
						self.class_block())

	def write_class(self):
		"Write the ZClass instance to a file on disk."
		file = open(self.product_directory+self.lname+'.py', 'w')
		file.write(self.compile_class())
		file.close()

	def dtml_add(self):
		"Return an Add.dtml text."
		if self.standard_headers == 1:
			block = open(self.dtml_directory+'Add_standard_headers.dtml', 'r').read()
		else:
			block = open(self.dtml_directory+'Add.dtml', 'r').read()
		attribmarkup = ""
		for attribute in self.attributes:
			block2 = open(self.dtml_directory+'Add_tr.dtml', 'r').read()
			attribmarkup = attribmarkup + block2 % (attribute[0],
													attribute[0],
													attribute[1])
		if self.nice_redirect == 1:
			action_tag = "<dtml-var \"make_action('"+str(self.name)+"', REQUEST)\">"
		else:
			action_tag = "manage_add" + str(self.name)
		if self.standard_headers == 1:
			return block % (action_tag,
							attribmarkup)
		else:
			return block % (self.name,
							action_tag,
							attribmarkup)

	def dtml_add2(self):
		"Return an Add2.dtml text."
		if self.standard_headers == 1:
			block = open(self.dtml_directory+'Add2_standard_headers.dtml', 'r').read()
			return block % (self.name,
							self.name,
							self.attriblist(3))
		else:
			block = open(self.dtml_directory+'Add2.dtml', 'r').read()
			return block % (self.name,
							self.name,
							self.attriblist(3))

	def dtml_index(self):
		"Return an Index.dtml text."
		if self.standard_headers == 1:
			block = open(self.dtml_directory+'Index_standard_headers.dtml', 'r').read()
			return block
		else:
			block = open(self.dtml_directory+'Index.dtml', 'r').read()
			return block

	def write_dtmlmethods(self):
		"Write Index and Add dtml files to disk."
		# Create the classnameAdd.dtml
		file = open(self.product_directory+self.lname+'Add.dtml', 'w')
		file.write(self.dtml_add())
		file.close
		# And then the classnameIndex.dtml
		file = open(self.product_directory+self.lname+'Index.dtml', 'w')
		file.write(self.dtml_index())
		file.close

		if self.nice_redirect == 1: 
			# Create the classnameAdd2.dtml
			file = open(self.product_directory+self.lname+'Add2.dtml', 'w')
			file.write(self.dtml_add2())
			file.close

	def add_class(self):
		"Setup a complete Zope Python Class."
		self.write_class()
		self.write_dtmlmethods()




